// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2007
// http://flock.com
//
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

const DEFAULT_DUMP_FILENAME = "rdf-data-debug.rdf";

const CC = Components.classes;
const CI = Components.interfaces;

/* from nspr's prio.h */
const PR_RDONLY      = 0x01;
const PR_WRONLY      = 0x02;
const PR_RDWR        = 0x04;
const PR_CREATE_FILE = 0x08;
const PR_APPEND      = 0x10;
const PR_TRUNCATE    = 0x20;
const PR_SYNC        = 0x40;
const PR_EXCL        = 0x80;

const RDFS = CC["@mozilla.org/rdf/rdf-service;1"]
             .getService(CI.nsIRDFService);
const NUM_TESTS = 100;

var _coop = CC["@flock.com/singleton;1"]
            .getService(CI.flockISingleton)
            .getSingleton("chrome://flock/content/common/load-faves-coop.js")
            .wrappedJSObject;

var ios = CC["@mozilla.org/network/io-service;1"].getService(CI.nsIIOService);

var loaded = false;
var mode;

function dump_ds(aDS, aOutfileName) {
  var ds = aDS;
  if (!ds) {
    ds = _coop.datasource;
  }

  var browser = document.getElementById("rdfviewBrowser");
  browser.setAttribute("src", "about:blank");

  var ser = CC["@mozilla.org/rdf/xml-serializer;1"]
            .createInstance(CI.nsIRDFXMLSerializer);
  ser.init(ds);
  ser.QueryInterface(CI.nsIRDFXMLSource);
  
  var outfile = getFullFileName(aOutfileName);
  var outStream = this.createOutStream(outfile);
  ser.Serialize(outStream);

  outStream.close();
  var fileUri = ios.newFileURI(outfile);
  browser.setAttribute("src", "view-source:" + fileUri.spec);
}

function getFullFileName(aOutfileName) {
  var profileDir = CC["@mozilla.org/file/directory_service;1"]
                   .getService(CI.nsIProperties)
                   .get("ProfD", CI.nsILocalFile);

  var outfileName = aOutfileName;
  if (!outfileName) {
    outfileName = DEFAULT_DUMP_FILENAME;
  }

  var outfile = profileDir.clone();
  outfile.append(outfileName);
  return outfile;
}

function createOutStream(aOutfile, mode) {
  if (mode != PR_APPEND) {
    try {
      aOutfile.remove(false);
    } catch (ex) {
    }
  }
  if (!aOutfile.exists()) {
    aOutfile.create(CI.nsIFile.NORMAL_FILE_TYPE, 0600);
  }
  var outStream = CC["@mozilla.org/network/file-output-stream;1"]
                  .createInstance(CI.nsIFileOutputStream);
  // 0600 == -rw------- file permissions
  outStream.init(aOutfile, PR_WRONLY | PR_CREATE_FILE | mode, 0600, 0);
  return outStream;
}

function onLoad() {
  if (!loaded ) {
    loaded = true;
    dump_ds();
  }
}

function unload() {
}

function rdf_test() {
  var browser = document.getElementById("rdfviewBrowser");
  browser.setAttribute("src", "about:blank");
  
  var outfile = getFullFileName("rdf-test.txt")
  // re-create the file the first time.
  var outStream = this.createOutStream(outfile, mode); 
  mode = PR_APPEND;
  
  addLabel(outStream, "Starting...............\n");

  testGetResource(outStream);  
  testGetResource(outStream);  

  selectionSort(outStream)
  selectionSort(outStream)
  
  testCoopCreateDeletes(outStream);
  testCoopCreateDeletes(outStream);

  testUnobservedAsserts(outStream);
  testUnobservedAsserts(outStream);

  var aDS = _coop.datasource; // GetDSBlocking("flock-test.rdf");
  testAsserts(outStream, aDS);
  testAsserts(outStream, aDS);

  startTestBlock();
  aDS.QueryInterface(CI.nsIRDFRemoteDataSource);
  aDS.Flush();
  addTestLog(outStream, "Done RDF flush.");

  outStream.close();
  var fileUri = ios.newFileURI(outfile);
  browser.setAttribute("src", "view-source:" + fileUri.spec);
}

function testGetResource(outStream) {
  var stubResource = "http://flock.com/rdf#MyMuchMuchLongerResourceWhichWillMakeMeHappy/";
  var bar = null;
  startTestBlock();
  for (var i = NUM_TESTS * 10; i > 0; i--) {
    bar = stubResource + i;
    var foo = RDFS.GetResource(bar);
  }
  addTestLog(outStream, "Finished looping GetResource test, " + NUM_TESTS*10 + " tests, longest full namespace was " + bar.length);
}

function testAsserts(outStream, aDS) {
  var myResource = RDFS.GetResource("f:parent");
  
  startTestBlock();
  var stubResource = "f:some";
  var stubPredicate = RDFS.GetResource("f:attitude");
  var bar = null;
  for (var i = NUM_TESTS * 10; i > 0; i--) {
    bar = stubResource + i;
    var foo = RDFS.GetResource(bar);
    aDS.Assert(myResource, stubPredicate, foo, true);
    aDS.Unassert(myResource, stubPredicate, foo);
  }
  addTestLog(outStream, "Finished looping Assert test, " 
      + NUM_TESTS * 10 + " tests, longest short namespace was " + bar.length);
  return aDS;
}


function testUnobservedAsserts(outStream) {
  var myResource = RDFS.GetResource("f:parent");
  var aDS = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
            .createInstance(Components.interfaces.nsIRDFDataSource);
  
  startTestBlock();
  var stubResource = "f:some";
  var stubPredicate = RDFS.GetResource("f:attitude");
  var bar = null;
  for (var i = NUM_TESTS * 10; i > 0; i--) {
    bar = stubResource + i;
    var foo = RDFS.GetResource(bar);
    aDS.Assert(myResource, stubPredicate, foo, true);
    aDS.Unassert(myResource, stubPredicate, foo);
  }
  addTestLog(outStream, "Finished looping Unobserved Assert test, " 
      + NUM_TESTS * 10 + " tests, longest short namespace was " + bar.length);
}

function testCoopCreateDeletes(outStream) {
  var aDS = _coop.datasource;
  startTestBlock();
  var foo = [];
  for (var i = NUM_TESTS; i > 0; i--)
  {
    foo.push(new _coop.Folder());
    foo.pop().destroy();
  }
  addTestLog(outStream, "Finished looping coop create/delete test, "
      + NUM_TESTS + " tests");
}

var beginTime = new Date().getTime();
function startTestBlock() {
  beginTime = new Date().getTime();
}

function addLabel(outStream, aStr) {
  outStream.write(aStr, aStr.length);
}

function addTestLog(outStream, aStr) {
  var endTime = new Date().getTime();
  var elapsedTime = ""+(endTime - beginTime);
  while(elapsedTime.length < 5){
    elapsedTime = " "+elapsedTime;
  }
  addLabel(outStream, "Elapsed time:"+ elapsedTime +": " + aStr +"\n");
}

// Circle class
function Circle(radius) {
  this.r = radius;
}
Circle.prototype.area = function() {
  return 3.14159 * this.r * this.r;
}
Circle.prototype.toString = function() {
  return "" + this.area();
}

function createRandomArray() {
  startTestBlock();
  var theArray = [];
  var size = NUM_TESTS;
  for(var i = 0; i< size; i++){
    var d1 = Math.random() * size;
    var c = new Circle(d1);
    theArray.push(c);
  }
  return theArray;
}
// Selection sort

function selectionSort(outStream) {
  startTestBlock();
  var theArray = createRandomArray();
  var n = theArray.length;
  var i = 0;
  while(i < n){
    var m = i;
    var j = i + 1;
    while(j < n){
	    if (theArray[j] > theArray[m])
		    m = j;
	    j = j + 1;
    }
    var t;
    t = theArray[i];
    theArray[i] = theArray[m];
    theArray[m] = t;
    i = i + 1;
  }
  addTestLog(outStream, "Finished selectionSort (no XPCOM calls), "
      + NUM_TESTS + " elements");
}
